

function ssw_fov_context, index, data , goes=goes, $
			      _extra=_extra, full_type=full_type, $
			      all_fov=all_fov, zbuffer=zbuffer, $
			      xsize=xsize, ysize=ysize, debug=debug, $
			      minframes=minframes, composite=composite, $
                              fdmap=fdmap, fd_type=fd_type,rr,gg,bb, $
			      fov=fov, pad_fov=pad_fov, center=center, $
			      rotate=rotate, oplotnar=oplotnar, offset=offset, $
			      xmargin=xmargin, ymargin=ymargin, $
			      midpoint=midpoint, ss=ss, grid=grid, fdroll=fdroll
;+
;   Name: ssw_fov_context
;
;   Purpose: generate a 'context' image for a reduced FOV or movie sequence
;
;   Input Parameters:
;      structure - index (of index,data pairings)
;      data  - the data cube (if index,data input)
;
;   Output:
;      Function returns the context image (overlay, GOES LC...)
;
;   Keyword Parameters:
;      fdmap - optional context map for overlays - if not passed, generated
;      fd_type - type of context map - default = 'eit171'
;                {'eit304','eit195','eit284','eit171'}  - EIT options
;                { 171,195}                             - TRACE options  
;                { sxt }                                - SXT optioins
;
;      fov - arcminutes FOV of full disk to include in composite
;            default is derived from [dXCEN, dYCEN]+pad
;
;      center - FOV center of full disk
;               defaults: FOV(index; ie, subfield) if only one FOV
;                         Sun Center if
;      oplotnar - if set, overplot NOAA AR
;      offset   - per OFFSET parameter in oplot_nar.pro 
;                 [offset the AR# from the location to see the action]
;      midpoint - if set, only overplot middle FOV (say)
;      extremes - if set, only overplot 1st and last
;      ss       - user supplied subset of fovs to overplot
;  
;   Calling Sequence:
;       IDL> context_image=ssw_fov_context(index, data [keywords] )
;  
;   History:
;       1-Jan-1999 - (Circa) - S.L.Freeland
;      14-Apr-1999 - S.L.Freeland add /OPLOTNAR and OFFSET keywords
;      25-May-1999 - S.L.Freeland add /MIDPOINT and SS keyword and function
;       7-Dec-1999 - S.L.Freeland - add FDROLL keyword and function (fd only)
;  
;   Method:
;      Use mapping SW for FOV overlay on FD image, optionally GOES/event
;
;   Calls:
;    SSW usual suspects (eit,sxt,trace)
;    index2fov, plot_map, drot_map, oplot_nar
;
;   Restrictions:
;      assumes desired context FD dbase online (eit, trace mosaics or SXT SFM)
;      (gsfc/lmsal/medoc/???)
;
;-
if n_params() lt 2 or (1-data_chk(index,/struct)) then begin 
        box_message,['Need "index,data" pair(s)', $
		     'IDL> img=ssw_fov_context(index,data [options] )']
        return,-1
endif	

dtemp=!d.name
debug=keyword_set(debug)
zbuffer=keyword_set(zbuffer) or get_logenv('ssw_batch') ne ''
oplotnar=keyword_set(oplotnar)
if not keyword_set(xsize) then xsize=750
if not keyword_set(ysize) then ysize=xsize

if not keyword_set(fd_type) then fd_type='eit171'      ; default
fdt=strlowcase(fd_type)
wave=str2number(fd_type)
if wave eq 0 then wave=171                             ; default is EIT 171
eit=strpos(fdt,'eit') ne -1
sxt=strpos(fdt,'sxt') ne -1
goes=keyword_set(goes)
tvlct,rr,gg,bb,/get
if n_elements(grid) eq 0 then grid=15
; -------------- get a context image --------------------------
case 1 of
  data_chk(fdmap,/struct):                ; user directly passed context map
  eit: begin
     time_window,index,time0,time1,day=2,/yohkoh
     estat=execute('fdf=eit_files(time0,time1,/full,wave=wave)')
      if fdf(0) eq '' then begin
         box_message,'No EIT context image close in time...'
	 return,-1
      endif 	
      box_message,'Using EIT , Wave: ' + strtrim(wave,2) + ' context image'
      ss=tim2dset(file2time(fdf,out='vms'),last_nelem(index))
      fdf=fdf(ss(0))
      read_eit,fdf,eindex,edata
      eit_prep, eindex, data=edata, fdindex, fddata
      fddata=sobel_scale(fdindex,fddata,hi=2000,sobel_weight=.05,minper=5)
  endcase
  sxt: begin
    box_message,'Using SXT context image'
    midtime=timegrid(index(0), $
	    hour=ssw_deltat(index(0),last_nelem(index),/hour)/2.)
    sfx=concat_dir('ydb','sfx')
    offday=ssw_deltat(anytim(midtime,/utc_int),$
               reltime(/now,out_style='utc_int'),/day)
    nrtfiles=concat_dir(sfx,(['sfx_newest.0000','sfx_last72.0000'])(offday le 3.5))

    estat=execute((['sfile=sxt_files(midtime,/sfd)','sfile=nrtfiles']) $
                    (offday le 31 and file_exist(nrtfiles)))
    if sfile(0) ne '' then begin
       read_sxt,sfile,-1,sindex
       ss=tim2dset(sindex,midtime)
       read_sxt,sfile,ss(0),fdindex,fddata
    endif else get_sfm, midtime, fdindex, fddata 
  endcase    
  else: begin
     estat=execute('fdf=trace_files(index(0),/synop,wave=wave)')
     read_trace,fdf,-1,fdindex,fddata
  endcase
endcase

fdindex=last_nelem(fdindex)                       ; assure only one
if data_chk(fddata,/nimage) gt 1 then fddata=last_nelem(fddata)

; make a full disk map object if not passed in
if not data_chk(fdmap,/struct) then index2map, fdindex, fddata, fdmap
wdef, zz, xsize,ysize, /zbuff
set_plot,'z'

if n_elements(charsize) eq 0 then $
    charsize=(([1.2,1.])(zbuffer)*(float(xsize)/1024)) + .2

; determine which fov(s) to overplot
nufov=n_elements(index)
ss=lindgen(nufov)                           ; default =ALL
case 1 of
   keyword_set(midpoint): ss=nufov/2             ; ~middle of seqence
   keyword_set(extremes): ss=[0,last_nelem(ss)]
   keyword_set(ss): fovss=ss                     ; user supplied
   else:                                         ; take default
endcase   

; --- get the FOV extremes / parameters based on all index ( "macro" FOV)

index2fov,index,east,west,north,south,/extreme, $   ; index -> FOV and size
     center_fov=center_fov, size_fov=size_fov

; --------- FOV size in arcminutes -------------
if n_elements(fov) eq 0 then begin                            ; use "macro" 
   if n_elements(pad_fov) eq 0 then pad_fov=max(2.*size_fov)  ; FOV
   fov=(size_fov+pad_fov)/60.         
endif 

if keyword_set(fdroll) then begin
   fdmap=rot_map(fdmap,fdroll)            ; de-roll
   fdmap.roll=0                           ; verabalize de-rolled
endif

; ------- where to center context ----------------------
if n_elements(center) eq 0 then center=center_fov             ; use "macro"
                                                              ; center
plot_map, fdmap, xsize=xsize, ysize=ysize,/square,grid=grid, _extra=_extra,/no16,$
         charsize=charsize, center=center, fov=fov, xmargin=xmargin, ymargin=ymargin, $
         /map2earth
;if oplotnar then oplot_nar,anytim(fdmap.time,/ecs),offset=offset

rotate=keyword_set(rotate)

; ------- -now overplot/composite subfields ----------------------
if n_elements(composite) eq 0 then composite=2        ; type of FD/FOV composite
nss=n_elements(ss)
for i=0,nss-1 do begin
   print,'Processing index >> ' + $
	  get_infox(index(ss(i)),'wave_len,naxis1,naxis2,xcen,ycen',fmt_tim='ecs')
   index2map, index(ss(i)), data(*,*,ss(i)), fovmap
   nmap=grid_map(fovmap,256,256)
   if rotate then begin
      message,/info,"rotating sub field with respect to context image"
      rotdur=ssw_deltat(nmap.time,fdmap.time,/hour)
      print,'fdt,fovt,dT(hours)',fdmap.time,nmap.time,rotdur
      nmap=drot_map(nmap,rotdur,/fast)
   endif     
   plot_map, nmap, composite=composite, /border, /no16, /map2earth
   if oplotnar then oplot_nar,anytim(nmap.time,/ecs), offset=offset
endfor  
out=tvrd()

; -------- optionally include goes lightcurve context --------
if keyword_set(goes) then begin
   wdef,xx,data_chk(out,/nx),data_chk(out,/nx)/4.,/zbuffer
   set_plot,'z'
   erase
   time_window, index, t0 , t1, percent=20             ; expand time range
   fmt_timer,index
   plot_goes,t0, t1, $
	      gcolor=20, color=130, gdata=gdata, charsize=charsize, $
	      ymargin=ymargin, xmargin=[5,.5], /fillnight, ncolor=40 
   if data_chk(gdata,/struct) then begin 
      evt_grid,index,ticklen=.1,tickpos=.8,color=150,thick=2,linestyle=0
      gout=!p.color-tvrd()
      out=[[gout],[out]]
   endif else message,/info,'NO GOES data between ' + t0 + ' and ' + t1
endif

set_plot,dtemp
return,out
end
